unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TForm1 = class(TForm)
	PaintBox: TPaintBox;
	procedure FormCreate(Sender: TObject);
	procedure FormDestroy(Sender: TObject);
	procedure PaintBoxMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
	procedure PaintBoxMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
	procedure PaintBoxMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
	procedure FormPaint(Sender: TObject);
  private
	IsPressed  : Boolean;
	PressedPos : TPoint;
  public
	SourBMP : TBitmap;
	DestBMP : TBitmap;
	BackBMP : TBitmap;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

function Min( A, B : Integer) : Integer;
begin
	if A < B then
		Result := A
	else
		Result := B;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
	SourBMP                    := TBitmap.Create;
	SourBMP.LoadFromFile( 'Test.bmp' );
	SourBMP.PixelFormat        := pf15Bit;

	DestBMP                    := TBitmap.Create;
	DestBMP.Width              := SourBMP.Width;
	DestBMP.Height             := SourBMP.Height;
	DestBMP.PixelFormat        := pf15Bit;
	DestBMP.Canvas.Pen.Color   := RGB( 179 div 2, 255, 143 div 2 );
	DestBMP.Canvas.Pen.Width   := 10;
	DestBMP.Canvas.Brush.Color := clBlack;
	DestBMP.Canvas.FillRect( DestBMP.Canvas.ClipRect );
	DestBMP.Canvas.Brush.Color := DestBMP.Canvas.Pen.Color;

	BackBMP                    := TBitmap.Create;
	BackBMP.Width              := SourBMP.Width;
	BackBMP.Height             := SourBMP.Height;
	BackBMP.PixelFormat        := pf15Bit;
	BackBMP.Canvas.Brush.Color := clBlack;
	BackBMP.Canvas.FillRect( DestBMP.Canvas.ClipRect );

	IsPressed := FALSE;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
	SourBMP.Free;
	DestBMP.Free;
	BackBMP.Free;
end;

procedure TForm1.FormPaint(Sender: TObject);
const
	ALPHA_FACTOR1 = 200;
	ALPHA_FACTOR2 = 256 - ALPHA_FACTOR1;
var
	x, y : Integer;
	pDest, pBack : PWord;
	R1, G1, B1 : Word;
	R2, G2, B2 : Word;
begin
	BackBMP.Canvas.Draw( 0, 0, SourBMP );

	for y := 0 to Pred( BackBMP.Height ) do begin
		pDest := PWord( DestBMP.ScanLine[y] );
		pBack := PWord( BackBMP.ScanLine[y] );
		for x := 0 to Pred( BackBMP.Width ) do begin
			if pDest^ > 0 then begin
				R1 := ( pDest^ shr 10 ) and $1F;
				G1 := ( pDest^ shr  5 ) and $1F;
				B1 := ( pDest^ shr  0 ) and $1F;
				R2 := ( pBack^ shr 10 ) and $1F;
				G2 := ( pBack^ shr  5 ) and $1F;
				B2 := ( pBack^ shr  0 ) and $1F;

				R1 := Min( ( R1 * ALPHA_FACTOR1 + R2 * ALPHA_FACTOR2 ) shr 8, 31 );
				G1 := Min( ( G1 * ALPHA_FACTOR1 + G2 * ALPHA_FACTOR2 ) shr 8, 31 );
				B1 := Min( ( B1 * ALPHA_FACTOR1 + B2 * ALPHA_FACTOR2 ) shr 8, 31 );

				pBack^ := ( R1 shl 10 ) or ( G1 shl 5 ) or B1;
			end;
			Inc( pDest );
			Inc( pBack );
		end;
	end;

	PaintBox.Canvas.Draw( 0, 0, BackBMP );
end;

procedure TForm1.PaintBoxMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	IsPressed  := TRUE;
	DestBMP.Canvas.MoveTo( X, Y );
end;

procedure TForm1.PaintBoxMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
	if IsPressed then begin
		DestBMP.Canvas.LineTo( X, Y );
		FormPaint( Self );
	end;
end;

procedure TForm1.PaintBoxMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	IsPressed := FALSE;
end;

end.
